#pragma once

#include <sys/stat.h>

#include "directory.h"

namespace cppfs
{

    /* directory::iterator */

    directory::iterator::iterator(const directory& p_owner, bool p_end)
        : _owner    (&p_owner)
        , _entry    ()
        , _dir      (nullptr, &closedir)
        , _dirent   (nullptr)
    {
        if (!p_end)
        {
            assert(_owner);
            auto s = _owner->path().str();
            _dir.reset(opendir(s.c_str()));
            if (!_dir)
                throw cppcore::error_exception("Unable to open directory", errno);
            next();
        }
    }

    directory::iterator::iterator(iterator&& p_other)
        : _owner    (std::move(p_other)._owner)
        , _entry    (std::move(p_other)._entry)
        , _dir      (std::move(p_other)._dir)
        , _dirent   (std::move(p_other)._dirent)
        { }

    directory::iterator::iterator(const iterator& p_other)
        : _owner    (p_other._owner)
        , _entry    ()
        , _dir      (nullptr, &closedir)
        , _dirent   (nullptr)
        { this->operator=(p_other); }

    directory::iterator& directory::iterator::operator=(iterator&& p_other)
    {
        _owner  = std::move(p_other)._owner;
        _entry  = std::move(p_other)._entry;
        _dir    = std::move(p_other)._dir;
        _dirent = std::move(p_other)._dirent;

        return *this;
    }

    directory::iterator& directory::iterator::operator=(const iterator& p_other)
    {
        _owner  = p_other._owner;
        _entry  = entry { };
        _dirent = nullptr;
        _dir.reset();

        assert(_owner);
        auto s = _owner->path().str();
        _dir.reset(opendir(s.c_str()));
        if (!_dir)
            throw cppcore::error_exception("Unable to open directory", errno);

        seekdir(_dir.get(), p_other.offset());
        next();

        return *this;
    }

    directory::iterator& directory::iterator::operator++()
    {
        next();
        return *this;
    }

    directory::iterator directory::iterator::operator++(int)
    {
        auto it = *this;
        next();
        return it;
    }

    bool directory::iterator::operator==(const iterator& p_other) const
    {
        return _owner
            && p_other._owner
            && _owner == p_other._owner
            && (   (!_dirent && !p_other._dirent)
                ||  offset() == p_other.offset());
    }

    bool directory::iterator::operator!=(const iterator& p_other) const
        { return !this->operator==(p_other); }

    directory::iterator::reference directory::iterator::operator* () const
    {
        assert(_dirent);
        return _entry;
    }

    directory::iterator::pointer directory::iterator::operator->() const
    {
        assert(_dirent);
        return &_entry;
    }

    void directory::iterator::next()
    {
        do
        {
            _dirent = readdir(_dir.get());
        }
        while (
                _dirent
            && (   (    _dirent->d_name[0] == '.'
                    &&  _dirent->d_name[1] == '.'
                    &&  _dirent->d_name[2] == '\0')
                || (    _dirent->d_name[0] == '.'
                    &&  _dirent->d_name[1] == '\0')));

        if (_dirent)
        {
            _entry.name = &_dirent->d_name[0];
            switch (_dirent->d_type)
            {
                case DT_BLK:  _entry.type = file_type::block_device;  break;
                case DT_CHR:  _entry.type = file_type::char_device;   break;
                case DT_DIR:  _entry.type = file_type::directory;     break;
                case DT_FIFO: _entry.type = file_type::named_pipe;    break;
                case DT_LNK:  _entry.type = file_type::symbolic_link; break;
                case DT_REG:  _entry.type = file_type::file;          break;
                case DT_SOCK: _entry.type = file_type::socket;        break;
                default:      _entry.type = file_type::unknown;       break;
            }
        }
    }

    long directory::iterator::offset() const
    {
        long ret = -1;
        if (_dir)
        {
            ret = telldir(_dir.get());
            if (ret < 0)
                throw cppcore::error_exception("Unable to get current position in directory", errno);
        }
        return ret;
    }

    /* directory */

    const directory& directory::create(
        bool                    p_parent,
        const file_permissions& p_perm) const
    {
        using namespace std::string_literals;

        auto perm = file_permissions::to_unix(p_perm);

        if (p_parent)
        {
            std::ostringstream os;

            auto& parts = _path.parts();
            for (auto& p : parts)
            {
                os << '/' << p;

                struct stat st;
                auto s = os.str();
                if (!stat(s.c_str(), &st) && S_ISDIR(st.st_mode))
                    continue;

                if (mkdir(s.c_str(), perm))
                    throw cppcore::error_exception("Unable to create directory: "s + s, errno);
            }
        }
        else
        {
            auto& s = _path.str();
            if (mkdir(s.c_str(), perm))
                throw cppcore::error_exception("Unable to create directory: "s + s, errno);
        }

        return *this;
    }

    void directory::remove_dir() const
    {
        using namespace std::string_literals;

        auto& p = _path.str();
        if (rmdir(p.c_str()))
            throw cppcore::error_exception("Unable to remove directory: "s + p, errno);
    }

}
