#pragma once

#include "path.h"

namespace cppfs
{

    /* path */

    path::path()
        : _status   (path_status::absoulte)
        , _path     ()
        , _parts    ()
        { }

    path::path(
        string_vector&& p_parts,
        path_status p_status)
        : _status   (p_status)
        , _path     ()
        , _parts    (std::move(p_parts))
        { }

    path::path(
        const string_vector& p_parts,
        path_status p_status)
        : _status   (p_status)
        , _path     ()
        , _parts    (p_parts)
        { }

    path::path(
        const std::initializer_list<std::string>& p_parts,
        path_status p_status)
        : _status   (p_status)
        , _path     ()
        , _parts    (std::begin(p_parts), std::end(p_parts))
        { }

    path_status path::status() const
        { return _status; }

    const std::string& path::str() const
    {
        if (_path.empty())
            _path = make_path();

        return _path;
    }

    const path::string_vector& path::parts() const
        { return _parts; }

    bool path::exsists() const
        { return type() != file_type::unknown; }

    bool path::is_file() const
    {
        auto t = type();
        return t != file_type::unknown
            && t != file_type::directory;
    }

    bool path::is_directory() const
        { return type() == file_type::directory; }

    path path::base() const
    {
        auto ret = normalize();

        if (!ret._parts.empty())
            ret._parts.pop_back();

        return ret;
    }

    path path::normalize() const
    {
        string_vector parts;
        if (_status != path_status::absoulte)
            parts = current().parts();

        for (auto& p : _parts)
        {
            if (p == ".." && !parts.empty())
                parts.pop_back();
            else if (p != ".")
                parts.emplace_back(p);
        }

        return path(std::move(parts), path_status::absoulte);
    }

    std::string path::extension() const
    {
        if (_parts.empty())
            return std::string();

        auto& s = _parts.back();
        auto  p = s.find_last_of('.');
        if (p == std::string::npos || p == 0)
            return std::string();

        return s.substr(p + 1);
    }

    path path::join(const path& other) const
    {
        switch (other._status)
        {
            case path_status::absoulte:
                return other;

            default:
                auto parts = _parts;
                for (auto& p : other.parts())
                    parts.emplace_back(p);
                return path(std::move(parts), _status);
        }
    }

}
